/*
  Library support for FlexRadio 6000 Rigs
  Copyright (C)2015 Vincenzo Stefanazzi - IW7DMH. All right reserved

  This library is free software; you can redistribute it and/or
  modify it under the terms of the CC BY-NC-SA 3.0 license.
  https://creativecommons.org/licenses/by-nc-sa/3.0/
  
  The license applies to all part of the library including the 
  examples and tools supplied with the library.
*/
#ifndef FlexRig_h
#define FlexRig_h

#include "Arduino.h"
#include <avr/dtostrf.h>
#include <Ethernet.h>
#include <SPI.h>
#include "Radio.h"
#include "Transmit.h"
#include "Interlock.h"
#include "Panadapter.h"
#include "Waterfall.h"
#include "Slice.h"
#include "Eq.h"

#define UDP_DISCOVERING_PORT 4992
#define UDP_TX_PACKET_MAX_SIZE 248
#define RESPONSE_PACKET_LIST_SIZE 25

//COMMAND (CIRCULAR) LIST
#define COMMAND_LIST_SIZE 20
#define COMMAND_LEGTH 80
#define UPDATE_VFO_TIMER 25  //millis


typedef struct ResponsePkt
{
  int id;                //Command id to match with
  int parserId;          //Message parser id
} rPacket;
 	
class FlexRig
{
  public:
    //properties
	byte ipAddress[4];       //IP Address
	char modelName[36];      //Model Name
    char serial[36];         //Serial Number
    char nickName[36];       //Nick Name
    char softVersion[32];    //Software Version
	char handle[8];          //Handle assigned to client
	int C;                   //Command id counter
	boolean connected=false; //Connection status
	//Embedded objects
	Radio radio;
	Transmit transmit;
	Interlock interlock;
	Panadapter panadapter[2];
	Waterfall waterfall[2];
	Slice slice[2];
	Eq eq;
    boolean vfo=false;
	int nMaxSlice=2;
	int activeSlice=0;
	int activePanadapter=0;
	int nMaxPanadapter=2;
	int activeWaterfall=0;
	int nMaxWaterfall=2;
	

	
    //methods
    FlexRig();
	void connect();
	//void setVFOA(float freq);
	void readEthernetData();
	void parseStatus(String msg);	
	void initBuffer();
	void send(String cmd);
	static FlexRig findAFlex(char* serial);
	void startReceiver();
	void stopReceiver();
	void updateObject(int objectId,int value,int activeSlice,int activePanadapter,int activeWaterfall);
	String removeFromCommandList();
	void addToCommandList(String entry);
	void sendAllCommands();
    
  private:
    //properties
	EthernetClient client;
	String buffer;             //tcp-ip buffer
	
	rPacket packetElement [RESPONSE_PACKET_LIST_SIZE]; //circular queque
	int front=0, rear=-1, size=0;   //pointers to response list array

	String CommandEntry[COMMAND_LIST_SIZE];
	int cFront=0, cRear=-1,cSize=0;   //pointers to commands circular list

	
	//methods
	void parseMessage(String msg);
	void send(String cmd, int parserId);
	//void send(String cmd);
	void configureClient();
	void addToResponseList(int id, int parserId);
	rPacket removeFromResponseList();
	void ping();
    static boolean isAFlex(char* model,char* serial,char* serialKey);
	
	//Parser Methods
	void parseReplay(String msg);
	void parseDummy(String msg);        //0
 	void parseAntennaList(String msg);  //1
    void parseMicList(String msg);	    //2

};

#endif
